@@ -14,24 +14,23 @@ pub struct DocOptions<'a> {
 
 pub fn doc(ws: &Workspace, options: &DocOptions) -> CargoResult<()> {
     let specs = options.compile_opts.spec.into_package_id_specs(ws)?;
-    let resolve = ops::resolve_ws_precisely(ws,
-                                            None,
-                                            options.compile_opts.features,
-                                            options.compile_opts.all_features,
-                                            options.compile_opts.no_default_features,
-                                            &specs)?;
+    let resolve = ops::resolve_ws_precisely(
+        ws,
+        None,
+        &options.compile_opts.features,
+        options.compile_opts.all_features,
+        options.compile_opts.no_default_features,
+        &specs,
+    )?;
     let (packages, resolve_with_overrides) = resolve;
 
-    if specs.is_empty() {
-        bail!("manifest path `{}` contains no package: The manifest is virtual, \
-               and the workspace has no members.",
-              ws.current_manifest().display())
-    };
-
-    let pkgs = specs.iter().map(|p| {
-        let pkgid = p.query(resolve_with_overrides.iter())?;
-        packages.get(pkgid)
-    }).collect::<CargoResult<Vec<_>>>()?;
+    let pkgs = specs
+        .iter()
+        .map(|p| {
+            let pkgid = p.query(resolve_with_overrides.iter())?;
+            packages.get(pkgid)
+        })
+        .collect::<CargoResult<Vec<_>>>()?;
 
     let mut lib_names = HashMap::new();
     let mut bin_names = HashMap::new();
@@ -39,18 +38,24 @@ pub fn doc(ws: &Workspace, options: &DocOptions) -> CargoResult<()> {
         for target in package.targets().iter().filter(|t| t.documented()) {
             if target.is_lib() {
                 if let Some(prev) = lib_names.insert(target.crate_name(), package) {
-                    bail!("The library `{}` is specified by packages `{}` and \
-                          `{}` but can only be documented once. Consider renaming \
-                          or marking one of the targets as `doc = false`.",
-                          target.crate_name(), prev, package);
-                }
-            } else {
-                if let Some(prev) = bin_names.insert(target.crate_name(), package) {
-                    bail!("The binary `{}` is specified by packages `{}` and \
-                          `{}` but can be documented only once. Consider renaming \
-                          or marking one of the targets as `doc = false`.",
-                          target.crate_name(), prev, package);
+                    bail!(
+                        "The library `{}` is specified by packages `{}` and \
+                         `{}` but can only be documented once. Consider renaming \
+                         or marking one of the targets as `doc = false`.",
+                        target.crate_name(),
+                        prev,
+                        package
+                    );
                 }
+            } else if let Some(prev) = bin_names.insert(target.crate_name(), package) {
+                bail!(
+                    "The binary `{}` is specified by packages `{}` and \
+                     `{}` but can be documented only once. Consider renaming \
+                     or marking one of the targets as `doc = false`.",
+                    target.crate_name(),
+                    prev,
+                    package
+                );
             }
         }
     }
@@ -59,10 +64,15 @@ pub fn doc(ws: &Workspace, options: &DocOptions) -> CargoResult<()> {
 
     if options.open_result {
         let name = if pkgs.len() > 1 {
-            bail!("Passing multiple packages and `open` is not supported.\n\
-                   Please re-run this command with `-p <spec>` where `<spec>` \
-                   is one of the following:\n  {}",
-                   pkgs.iter().map(|p| p.name()).collect::<Vec<_>>().join("\n  "));
+            bail!(
+                "Passing multiple packages and `open` is not supported.\n\
+                 Please re-run this command with `-p <spec>` where `<spec>` \
+                 is one of the following:\n  {}",
+                pkgs.iter()
+                    .map(|p| p.name().to_inner())
+                    .collect::<Vec<_>>()
+                    .join("\n  ")
+            );
         } else if pkgs.len() == 1 {
             pkgs[0].name().replace("-", "_")
         } else {
@@ -76,7 +86,7 @@ pub fn doc(ws: &Workspace, options: &DocOptions) -> CargoResult<()> {
         // nothing we can do about it and otherwise if it's getting overwritten
         // then that's also ok!
         let mut target_dir = ws.target_dir();
-        if let Some(triple) = options.compile_opts.target {
+        if let Some(ref triple) = options.compile_opts.target {
             target_dir.push(Path::new(triple).file_stem().unwrap());
         }
         let path = target_dir.join("doc").join(&name).join("index.html");
@@ -87,8 +97,7 @@ pub fn doc(ws: &Workspace, options: &DocOptions) -> CargoResult<()> {
             match open_docs(&path) {
                 Ok(m) => shell.status("Launching", m)?,
                 Err(e) => {
-                    shell.warn(
-                            "warning: could not determine a browser to open docs with, tried:")?;
+                    shell.warn("warning: could not determine a browser to open docs with, tried:")?;
                     for method in e {
                         shell.warn(format!("\t{}", method))?;
                     }
