@@ -1,7 +1,8 @@
-use std::ffi::{OsString, OsStr};
+use std::ffi::{OsStr, OsString};
 
 use ops::{self, Compilation};
-use util::{self, CargoResult, CargoTestError, Test, ProcessError};
+use util::{self, CargoTestError, ProcessError, Test};
+use util::errors::CargoResult;
 use core::Workspace;
 
 pub struct TestOptions<'a> {
@@ -11,15 +12,18 @@ pub struct TestOptions<'a> {
     pub only_doc: bool,
 }
 
-pub fn run_tests(ws: &Workspace,
-                 options: &TestOptions,
-                 test_args: &[String]) -> CargoResult<Option<CargoTestError>> {
+pub fn run_tests(
+    ws: &Workspace,
+    options: &TestOptions,
+    test_args: &[String],
+) -> CargoResult<Option<CargoTestError>> {
     let compilation = compile_tests(ws, options)?;
 
     if options.no_run {
-        return Ok(None)
+        return Ok(None);
     }
     let (test, mut errors) = if options.only_doc {
+        assert!(options.compile_opts.filter.is_specific());
         run_doc_tests(options, test_args, &compilation)?
     } else {
         run_unit_tests(options, test_args, &compilation)?
@@ -27,36 +31,39 @@ pub fn run_tests(ws: &Workspace,
 
     // If we have an error and want to fail fast, return
     if !errors.is_empty() && !options.no_fail_fast {
-        return Ok(Some(CargoTestError::new(test, errors)))
+        return Ok(Some(CargoTestError::new(test, errors)));
     }
 
     // If a specific test was requested or we're not running any tests at all,
     // don't run any doc tests.
-    if let ops::CompileFilter::Only { .. } = options.compile_opts.filter {
+    if options.compile_opts.filter.is_specific() {
         match errors.len() {
             0 => return Ok(None),
-            _ => return Ok(Some(CargoTestError::new(test, errors)))
+            _ => return Ok(Some(CargoTestError::new(test, errors))),
         }
     }
 
     let (doctest, docerrors) = run_doc_tests(options, test_args, &compilation)?;
+    let test = if docerrors.is_empty() { test } else { doctest };
     errors.extend(docerrors);
     if errors.is_empty() {
         Ok(None)
     } else {
-        Ok(Some(CargoTestError::new(doctest, errors)))
+        Ok(Some(CargoTestError::new(test, errors)))
     }
 }
 
-pub fn run_benches(ws: &Workspace,
-                   options: &TestOptions,
-                   args: &[String]) -> CargoResult<Option<CargoTestError>> {
+pub fn run_benches(
+    ws: &Workspace,
+    options: &TestOptions,
+    args: &[String],
+) -> CargoResult<Option<CargoTestError>> {
     let mut args = args.to_vec();
     args.push("--bench".to_string());
     let compilation = compile_tests(ws, options)?;
 
     if options.no_run {
-        return Ok(None)
+        return Ok(None);
     }
     let (test, errors) = run_unit_tests(options, &args, &compilation)?;
     match errors.len() {
@@ -65,21 +72,23 @@ pub fn run_benches(ws: &Workspace,
     }
 }
 
-fn compile_tests<'a>(ws: &Workspace<'a>,
-                     options: &TestOptions<'a>)
-                     -> CargoResult<Compilation<'a>> {
+fn compile_tests<'a>(
+    ws: &Workspace<'a>,
+    options: &TestOptions<'a>,
+) -> CargoResult<Compilation<'a>> {
     let mut compilation = ops::compile(ws, &options.compile_opts)?;
-    compilation.tests.sort_by(|a, b| {
-        (a.0.package_id(), &a.1, &a.2).cmp(&(b.0.package_id(), &b.1, &b.2))
-    });
+    compilation
+        .tests
+        .sort_by(|a, b| (a.0.package_id(), &a.1, &a.2).cmp(&(b.0.package_id(), &b.1, &b.2)));
     Ok(compilation)
 }
 
 /// Run the unit and integration tests of a project.
-fn run_unit_tests(options: &TestOptions,
-                  test_args: &[String],
-                  compilation: &Compilation)
-                  -> CargoResult<(Test, Vec<ProcessError>)> {
+fn run_unit_tests(
+    options: &TestOptions,
+    test_args: &[String],
+    compilation: &Compilation,
+) -> CargoResult<(Test, Vec<ProcessError>)> {
     let config = options.compile_opts.config;
     let cwd = options.compile_opts.config.cwd();
 
@@ -92,27 +101,50 @@ fn run_unit_tests(options: &TestOptions,
         };
         let mut cmd = compilation.target_process(exe, pkg)?;
         cmd.args(test_args);
-        config.shell().concise(|shell| {
-            shell.status("Running", to_display.display().to_string())
-        })?;
-        config.shell().verbose(|shell| {
-            shell.status("Running", cmd.to_string())
-        })?;
+        config
+            .shell()
+            .concise(|shell| shell.status("Running", to_display.display().to_string()))?;
+        config
+            .shell()
+            .verbose(|shell| shell.status("Running", cmd.to_string()))?;
 
-        if let Err(e) = cmd.exec() {
-            errors.push(e);
-            if !options.no_fail_fast {
-                return Ok((Test::UnitTest(kind.clone(), test.clone()), errors))
+        let result = cmd.exec();
+
+        match result {
+            Err(e) => {
+                let e = e.downcast::<ProcessError>()?;
+                errors.push((kind.clone(), test.clone(), pkg.name().to_string(), e));
+                if !options.no_fail_fast {
+                    break;
+                }
             }
+            Ok(()) => {}
         }
     }
-    Ok((Test::Multiple, errors))
+
+    if errors.len() == 1 {
+        let (kind, name, pkg_name, e) = errors.pop().unwrap();
+        Ok((
+            Test::UnitTest {
+                kind,
+                name,
+                pkg_name,
+            },
+            vec![e],
+        ))
+    } else {
+        Ok((
+            Test::Multiple,
+            errors.into_iter().map(|(_, _, _, e)| e).collect(),
+        ))
+    }
 }
 
-fn run_doc_tests(options: &TestOptions,
-                 test_args: &[String],
-                 compilation: &Compilation)
-                 -> CargoResult<(Test, Vec<ProcessError>)> {
+fn run_doc_tests(
+    options: &TestOptions,
+    test_args: &[String],
+    compilation: &Compilation,
+) -> CargoResult<(Test, Vec<ProcessError>)> {
     let mut errors = Vec::new();
     let config = options.compile_opts.config;
 
@@ -122,26 +154,41 @@ fn run_doc_tests(options: &TestOptions,
     }
 
     let libs = compilation.to_doc_test.iter().map(|package| {
-        (package, package.targets().iter().filter(|t| t.doctested())
-                         .map(|t| (t.src_path(), t.name(), t.crate_name())))
+        (
+            package,
+            package
+                .targets()
+                .iter()
+                .filter(|t| t.doctested())
+                .map(|t| (t.src_path(), t.name(), t.crate_name())),
+        )
     });
 
     for (package, tests) in libs {
         for (lib, name, crate_name) in tests {
             config.shell().status("Doc-tests", name)?;
             let mut p = compilation.rustdoc_process(package)?;
-            p.arg("--test").arg(lib)
-             .arg("--crate-name").arg(&crate_name);
+            p.arg("--test")
+                .arg(lib)
+                .arg("--crate-name")
+                .arg(&crate_name);
 
             for &rust_dep in &[&compilation.deps_output] {
                 let mut arg = OsString::from("dependency=");
                 arg.push(rust_dep);
                 p.arg("-L").arg(arg);
             }
+
             for native_dep in compilation.native_dirs.iter() {
                 p.arg("-L").arg(native_dep);
             }
 
+            for &host_rust_dep in &[&compilation.host_deps_output] {
+                let mut arg = OsString::from("dependency=");
+                arg.push(host_rust_dep);
+                p.arg("-L").arg(arg);
+            }
+
             for arg in test_args {
                 p.arg("--test-args").arg(arg);
             }
@@ -164,9 +211,8 @@ fn run_doc_tests(options: &TestOptions,
                 // dynamically as well, causing problems. As a result we only
                 // pass `--extern` for rlib deps and skip out on all other
                 // artifacts.
-                if lib.extension() != Some(OsStr::new("rlib")) &&
-                   !target.for_host() {
-                    continue
+                if lib.extension() != Some(OsStr::new("rlib")) && !target.for_host() {
+                    continue;
                 }
                 let mut arg = OsString::from(target.crate_name());
                 arg.push("=");
@@ -174,10 +220,15 @@ fn run_doc_tests(options: &TestOptions,
                 p.arg("--extern").arg(&arg);
             }
 
-            config.shell().verbose(|shell| {
-                shell.status("Running", p.to_string())
-            })?;
+            if let Some(flags) = compilation.rustdocflags.get(package.package_id()) {
+                p.args(flags);
+            }
+
+            config
+                .shell()
+                .verbose(|shell| shell.status("Running", p.to_string()))?;
             if let Err(e) = p.exec() {
+                let e = e.downcast::<ProcessError>()?;
                 errors.push(e);
                 if !options.no_fail_fast {
                     return Ok((Test::Doc, errors));
